/* new memory functions */

#define MEMTBLSIZE 768
#undef DEBUG

#include <stdio.h>

extern int serv;

unsigned memused=0;

#ifdef DEBUG
struct {
  void *ptr;
  int size;
  char file[20];
  int line;
} memtbl[MEMTBLSIZE];
#endif

/* initialize the memory structure */
void init_mem()
{
#ifdef DEBUG
  int i;
  for (i=0; i<MEMTBLSIZE; i++) memtbl[i].ptr=NULL;
#endif
}

/* tell someone the gory memory details */
void tell_mem_status(nick)
char *nick;
{
#ifdef DEBUG
  int i,tot=0; float per;
  for (i=0; i<MEMTBLSIZE; i++) if (memtbl[i].ptr!=NULL) tot++;
  per=((tot*1.0)/(MEMTBLSIZE*1.0))*100.0;
  mprintf(serv,"PRIVMSG %s :Memory table usage: %d/%d (%.1f%% full)\n",nick,
	  tot,MEMTBLSIZE,per);
#endif
  mprintf(serv,"PRIVMSG %s :Total memory allocated = %d bytes\n",nick,memused);
}

void tell_mem_status_dcc(z)
int z;
{
  int exp,i,tot=0; float per;
  exp=expected_memory();  /* in io.c ? */
#ifdef DEBUG
  for (i=0; i<MEMTBLSIZE; i++) if (memtbl[i].ptr!=NULL) tot++;
  per=((tot*1.0)/(MEMTBLSIZE*1.0))*100.0;
  tprintf(z,"Memory table usage: %d/%d (%.1f%% full)\n",tot,MEMTBLSIZE,per);
#endif
  per=((exp*1.0)/(memused*1.0))*100.0;
  if (per!=100.0)
    tprintf(z,"Memory fault: only accounting for %d/%d (%.1f%%)\n",
	    exp,memused,per);
#ifdef DEBUG
  tprintf(z,"Memory table itself occupies an additional %d bytes static\n",
	  MEMTBLSIZE*32);
#endif
}

void debug_mem_to_dcc(z)
int z;
{
#ifdef DEBUG
  int i; char sofar[81],s[81],fn[20];
  sofar[0]=0;
  for (i=0; i<MEMTBLSIZE; i++) if (memtbl[i].ptr!=NULL) {
    strcpy(fn,memtbl[i].file); fn[7]=0;
    sprintf(s,"%-7s/%-3d (%04X) ",fn,memtbl[i].line,memtbl[i].size);
    strcat(sofar,s);
    if (strlen(sofar)>60) {
      sofar[strlen(sofar)-1]=0;
      tprintf(z,"%s\n",sofar);
      sofar[0]=0;
    }
  }
  if (sofar[0]) {
    sofar[strlen(sofar)-1]=0;
    tprintf(z,"%s\n",sofar);
  }
  tprintf(z,"--- End of debug memory list.\n");
#else
  tprintf(z,"Compiled without extensive memory debugging (sorry).\n");
#endif
}

void *n_malloc(size,file,line)
int size,line; char *file;
{
  void *x; int i;
#ifdef DEBUG
  x=(void *)malloc(size);
  if (x==NULL) {
    console("*** FAILED MALLOC %s (%d)",file,line);
    return NULL;
  }
  for (i=0; (i<MEMTBLSIZE) && (memtbl[i].ptr!=NULL); i++);
  if (i==MEMTBLSIZE) {
    console("*** MEMORY TABLE FULL: %s (%d)",file,line);
    return x;
  }
  memtbl[i].ptr=x; memtbl[i].line=line; memtbl[i].size=size;
  strcpy(memtbl[i].file,file);
#else
  x=(void *)malloc(size+4); *(int *)(x)=size; 
  (char *)x+=4;
#endif
  memused+=size;
  return x;
}

void n_free(ptr,file,line)
void *ptr; char *file; int line;
{
  int i;
#ifdef DEBUG
  if (ptr==NULL) {
    console("*** ATTEMPTING TO FREE NULL PTR: %s (%d)",file,line);
    return;
  }
  for (i=0; (i<MEMTBLSIZE) && (memtbl[i].ptr!=ptr); i++);
  if (i==MEMTBLSIZE) {
    console("*** ATTEMTPING TO FREE NON-MALLOC'D PTR: %s (%d)",file,line);
    return;
  }
  memtbl[i].ptr=NULL;
  memused-=memtbl[i].size; free(ptr);
#else
  memused-=*(int *)((char *)(ptr)-4); free((char *)(ptr)-4);
#endif
}
